from collections.abc import Callable
from dataclasses import dataclass, field
from functools import cached_property
from pathlib import Path

from fluent.runtime import (
    FluentLocalization,
    FluentResourceLoader,
)
from telegrinder.node import IsNode


@dataclass
class FluentConfig:
    folder: Path | str
    source: IsNode
    default_locale: str = "en"
    replace_underscore: bool = True
    functions: dict[str, Callable] = field(default_factory=dict)

    def __post_init__(self) -> None:
        if not isinstance(self.folder, Path):
            self.folder = Path(self.folder)

    @cached_property
    def loaders(self) -> dict[str, FluentLocalization]:
        result: dict[str, FluentLocalization] = {}

        for locale_dir in self.folder.iterdir(): # type: ignore
            if not locale_dir.is_dir():
                continue

            ftl_files = [
                    str(p.relative_to(locale_dir))
                    for p in locale_dir.rglob("*.ftl")
                ]
            if not ftl_files:
                continue

            loader = FluentResourceLoader(str(self.folder / "{locale}")) # type: ignore
            localization = FluentLocalization(
                locales=[locale_dir.name],
                resource_ids=ftl_files,
                resource_loader=loader,
                functions=self.functions,
            )

            result[locale_dir.name] = localization

        return result

    def get_translator(self, locale: str) -> FluentLocalization:
        return self.loaders.get(locale, self.loaders[self.default_locale])



import typing

from telegrinder.node.i18n import ABCTranslator

from fluent_telegrinder.config import FluentConfig


class FluentTranslator(ABCTranslator):
    config: typing.ClassVar[FluentConfig]

    def __class_getitem__(cls, config: FluentConfig, /) -> typing.Any:
        return type(cls.__name__, (cls,), {"config": config})

    @classmethod
    def configure(cls, config: FluentConfig, /) -> None:
        cls.config = config

    @property
    def message_id(self) -> str:
        if self.config.replace_underscore:
            return (
                self.separator
                .join("-".join(key.split("_")) for key in self._keys)
            )
        return self.separator.join(self._keys)

    def translate(self, message_id: str, **context: typing.Any) -> str:
        return (
            self.config
            .get_translator(self.locale)
            .format_value(message_id, context)
        )


type Translator = FluentTranslator



from telegrinder.node import UserSource, scalar_node

from fluent_telegrinder.i18n import FluentTranslator


@scalar_node
class DefaultLocaleSource:
    @classmethod
    async def compose(cls) -> str:
        return FluentTranslator.config.default_locale


@scalar_node
class UserLanguageSource:
    @classmethod
    async def compose(cls, user: UserSource) -> str:
        return (
            user.language_code
            .unwrap_or(FluentTranslator.config.default_locale)
        )
